@@ -231,12 +231,12 @@ class LocalClient(object):
         Return the information about a given job
         '''
         log.debug('Checking whether jid {0} is still running'.format(jid))
-        timeout = self.opts['gather_job_timeout']
+        timeout = int(kwargs.get('gather_job_timeout', self.opts['gather_job_timeout']))
 
         pub_data = self.run_job(tgt,
                                 'saltutil.find_job',
                                 arg=[jid],
-                                expr_form=tgt_type,
+                                tgt_type=tgt_type,
                                 timeout=timeout,
                                 **kwargs
                                )
@@ -291,7 +291,7 @@ class LocalClient(object):
             tgt,
             fun,
             arg=(),
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             timeout=None,
             jid='',
@@ -312,6 +312,15 @@ class LocalClient(object):
             >>> local.run_job('*', 'test.sleep', [300])
             {'jid': '20131219215650131543', 'minions': ['jerry']}
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
 
         try:
@@ -319,7 +328,7 @@ class LocalClient(object):
                 tgt,
                 fun,
                 arg,
-                expr_form,
+                tgt_type,
                 ret,
                 jid=jid,
                 timeout=self._get_timeout(timeout),
@@ -336,13 +345,16 @@ class LocalClient(object):
 
         return self._check_pub_data(pub_data)
 
+    def gather_minions(self, tgt, expr_form):
+        return salt.utils.minions.CkMinions(self.opts).check_minions(tgt, tgt_type=expr_form)
+
     @tornado.gen.coroutine
     def run_job_async(
             self,
             tgt,
             fun,
             arg=(),
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             timeout=None,
             jid='',
@@ -364,6 +376,15 @@ class LocalClient(object):
             >>> local.run_job_async('*', 'test.sleep', [300])
             {'jid': '20131219215650131543', 'minions': ['jerry']}
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
 
         try:
@@ -371,7 +392,7 @@ class LocalClient(object):
                   tgt,
                   fun,
                   arg,
-                  expr_form,
+                  tgt_type,
                   ret,
                   jid=jid,
                   timeout=self._get_timeout(timeout),
@@ -394,7 +415,7 @@ class LocalClient(object):
             tgt,
             fun,
             arg=(),
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             jid='',
             kwarg=None,
@@ -412,11 +433,20 @@ class LocalClient(object):
             >>> local.cmd_async('*', 'test.sleep', [300])
             '20131219215921857715'
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
         pub_data = self.run_job(tgt,
                                 fun,
                                 arg,
-                                expr_form,
+                                tgt_type,
                                 ret,
                                 jid=jid,
                                 **kwargs)
@@ -430,7 +460,7 @@ class LocalClient(object):
             tgt,
             fun,
             arg=(),
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             kwarg=None,
             sub=3,
@@ -450,8 +480,20 @@ class LocalClient(object):
             >>> SLC.cmd_subset('*', 'test.ping', sub=1)
             {'jerry': True}
         '''
-        minion_ret = self.cmd(tgt, 'sys.list_functions', expr_form=expr_form, **kwargs)
-        minions = minion_ret.keys()
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
+        minion_ret = self.cmd(tgt,
+                              'sys.list_functions',
+                              tgt_type=tgt_type,
+                              **kwargs)
+        minions = list(minion_ret)
         random.shuffle(minions)
         f_tgt = []
         for minion in minions:
@@ -466,7 +508,7 @@ class LocalClient(object):
                 f_tgt,
                 fun,
                 arg,
-                expr_form='list',
+                tgt_type='list',
                 ret=ret,
                 kwarg=kwarg,
                 progress=progress,
@@ -477,7 +519,7 @@ class LocalClient(object):
             tgt,
             fun,
             arg=(),
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             kwarg=None,
             batch='10%',
@@ -501,31 +543,56 @@ class LocalClient(object):
             {'dave': {...}}
             {'stewart': {...}}
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         import salt.cli.batch
         arg = salt.utils.args.condition_input(arg, kwarg)
         opts = {'tgt': tgt,
                 'fun': fun,
                 'arg': arg,
-                'expr_form': expr_form,
+                'tgt_type': tgt_type,
                 'ret': ret,
                 'batch': batch,
                 'raw': kwargs.get('raw', False)}
+
+        if 'timeout' in kwargs:
+            opts['timeout'] = kwargs['timeout']
+        if 'gather_job_timeout' in kwargs:
+            opts['gather_job_timeout'] = kwargs['gather_job_timeout']
+
+        eauth = {}
+        if 'eauth' in kwargs:
+            eauth['eauth'] = kwargs.pop('eauth')
+        if 'username' in kwargs:
+            eauth['username'] = kwargs.pop('username')
+        if 'password' in kwargs:
+            eauth['password'] = kwargs.pop('password')
+        if 'token' in kwargs:
+            eauth['token'] = kwargs.pop('token')
+
         for key, val in six.iteritems(self.opts):
             if key not in opts:
                 opts[key] = val
-        batch = salt.cli.batch.Batch(opts, quiet=True)
+        batch = salt.cli.batch.Batch(opts, eauth=eauth, quiet=True)
         for ret in batch.run():
             yield ret
 
-    def cmd(
-            self,
+    def cmd(self,
             tgt,
             fun,
             arg=(),
             timeout=None,
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             jid='',
+            full_return=False,
             kwarg=None,
             **kwargs):
         '''
@@ -566,7 +633,7 @@ class LocalClient(object):
                 ])
 
         :param tgt: Which minions to target for the execution. Default is shell
-            glob. Modified by the ``expr_form`` option.
+            glob. Modified by the ``tgt_type`` option.
         :type tgt: string or list
 
         :param fun: The module and function to call on the specified minions of
@@ -592,7 +659,7 @@ class LocalClient(object):
         :param timeout: Seconds to wait after the last minion returns but
             before all minions return.
 
-        :param expr_form: The type of ``tgt``. Allowed values:
+        :param tgt_type: The type of ``tgt``. Allowed values:
 
             * ``glob`` - Bash glob completion - Default
             * ``pcre`` - Perl style regular expression
@@ -604,6 +671,10 @@ class LocalClient(object):
             * ``nodegroup`` - Match on nodegroup
             * ``range`` - Use a Range server for matching
             * ``compound`` - Pass a compound match string
+            * ``ipcidr`` - Match based on Subnet (CIDR notation) or IPv4 address.
+
+            .. versionchanged:: 2017.7.0
+                Renamed from ``expr_form`` to ``tgt_type``
 
         :param ret: The returner to use. The value passed can be single
             returner, or a comma delimited list of returners to call in order
@@ -611,6 +682,9 @@ class LocalClient(object):
 
         :param kwarg: A dictionary with keyword arguments for the function.
 
+        :param full_return: Output the job return only (default) or the full
+            return including exit code and other job metadata.
+
         :param kwargs: Optional keyword arguments.
             Authentication credentials may be passed when using
             :conf_master:`external_auth`.
@@ -624,6 +698,15 @@ class LocalClient(object):
             minion ID. A compound command will return a sub-dictionary keyed by
             function name.
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
         was_listening = self.event.cpub
 
@@ -631,7 +714,7 @@ class LocalClient(object):
             pub_data = self.run_job(tgt,
                                     fun,
                                     arg,
-                                    expr_form,
+                                    tgt_type,
                                     ret,
                                     timeout,
                                     jid,
@@ -647,12 +730,16 @@ class LocalClient(object):
                     pub_data['minions'],
                     self._get_timeout(timeout),
                     tgt,
-                    expr_form,
+                    tgt_type,
                     **kwargs):
 
                 if fn_ret:
                     for mid, data in six.iteritems(fn_ret):
-                        ret[mid] = data.get('ret', {})
+                        ret[mid] = (data if full_return
+                                else data.get('ret', {}))
+
+            for failed in list(set(pub_data['minions']) ^ set(ret)):
+                ret[failed] = False
             return ret
         finally:
             if not was_listening:
@@ -664,7 +751,7 @@ class LocalClient(object):
             fun,
             arg=(),
             timeout=None,
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             verbose=False,
             kwarg=None,
@@ -680,6 +767,15 @@ class LocalClient(object):
         :param verbose: Print extra information about the running command
         :returns: A generator
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
         was_listening = self.event.cpub
 
@@ -688,7 +784,7 @@ class LocalClient(object):
                 tgt,
                 fun,
                 arg,
-                expr_form,
+                tgt_type,
                 ret,
                 timeout,
                 listen=True,
@@ -703,7 +799,7 @@ class LocalClient(object):
                             self.pub_data['minions'],
                             self._get_timeout(timeout),
                             tgt,
-                            expr_form,
+                            tgt_type,
                             verbose,
                             progress,
                             **kwargs):
@@ -733,7 +829,7 @@ class LocalClient(object):
             fun,
             arg=(),
             timeout=None,
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             kwarg=None,
             **kwargs):
@@ -754,6 +850,15 @@ class LocalClient(object):
             {'dave': {'ret': True}}
             {'stewart': {'ret': True}}
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
         was_listening = self.event.cpub
 
@@ -762,7 +867,7 @@ class LocalClient(object):
                 tgt,
                 fun,
                 arg,
-                expr_form,
+                tgt_type,
                 ret,
                 timeout,
                 listen=True,
@@ -777,7 +882,7 @@ class LocalClient(object):
                                                     pub_data['minions'],
                                                     timeout=self._get_timeout(timeout),
                                                     tgt=tgt,
-                                                    tgt_type=expr_form,
+                                                    tgt_type=tgt_type,
                                                     **kwargs):
                     if not fn_ret:
                         continue
@@ -793,7 +898,7 @@ class LocalClient(object):
             fun,
             arg=(),
             timeout=None,
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             kwarg=None,
             show_jid=False,
@@ -821,6 +926,15 @@ class LocalClient(object):
             None
             {'stewart': {'ret': True}}
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
         was_listening = self.event.cpub
 
@@ -829,7 +943,7 @@ class LocalClient(object):
                 tgt,
                 fun,
                 arg,
-                expr_form,
+                tgt_type,
                 ret,
                 timeout,
                 listen=True,
@@ -842,11 +956,11 @@ class LocalClient(object):
                                                     pub_data['minions'],
                                                     timeout=timeout,
                                                     tgt=tgt,
-                                                    tgt_type=expr_form,
+                                                    tgt_type=tgt_type,
                                                     block=False,
                                                     **kwargs):
                     if fn_ret and any([show_jid, verbose]):
-                        for minion in fn_ret.keys():
+                        for minion in fn_ret:
                             fn_ret[minion]['jid'] = pub_data['jid']
                     yield fn_ret
 
@@ -861,7 +975,7 @@ class LocalClient(object):
             fun,
             arg=(),
             timeout=None,
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             verbose=False,
             kwarg=None,
@@ -869,6 +983,15 @@ class LocalClient(object):
         '''
         Execute a salt command and return
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         arg = salt.utils.args.condition_input(arg, kwarg)
         was_listening = self.event.cpub
 
@@ -877,7 +1000,7 @@ class LocalClient(object):
                 tgt,
                 fun,
                 arg,
-                expr_form,
+                tgt_type,
                 ret,
                 timeout,
                 listen=True,
@@ -890,7 +1013,7 @@ class LocalClient(object):
                                                       pub_data['minions'],
                                                       timeout,
                                                       tgt,
-                                                      expr_form,
+                                                      tgt_type,
                                                       verbose))
         finally:
             if not was_listening:
@@ -911,6 +1034,15 @@ class LocalClient(object):
 
         :returns: all of the information for the JID
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         if verbose:
             msg = 'Executing job with jid {0}'.format(jid)
             print(msg)
@@ -981,6 +1113,15 @@ class LocalClient(object):
 
         :returns: all of the information for the JID
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         if not isinstance(minions, set):
             if isinstance(minions, six.string_types):
                 minions = set([minions])
@@ -989,6 +1130,7 @@ class LocalClient(object):
 
         if timeout is None:
             timeout = self.opts['timeout']
+        gather_job_timeout = int(kwargs.get('gather_job_timeout', self.opts['gather_job_timeout']))
         start = int(time.time())
 
         # timeouts per minion, id_ -> timeout time
@@ -1089,7 +1231,7 @@ class LocalClient(object):
                     jinfo_iter = []
                 else:
                     jinfo_iter = self.get_returns_no_block('salt/job/{0}'.format(jinfo['jid']))
-                timeout_at = time.time() + self.opts['gather_job_timeout']
+                timeout_at = time.time() + gather_job_timeout
                 # if you are a syndic, wait a little longer
                 if self.opts['order_masters']:
                     timeout_at += self.opts.get('syndic_wait', 1)
@@ -1105,8 +1247,17 @@ class LocalClient(object):
                         minions.remove(raw['data']['id'])
                         break
                 except KeyError as exc:
-                    # This is a safe pass. We're just using the try/except to avoid having to deep-check for keys
-                    log.debug('Passing on saltutil error. This may be an error in saltclient. {0}'.format(exc))
+                    # This is a safe pass. We're just using the try/except to
+                    # avoid having to deep-check for keys.
+                    missing_key = exc.__str__().strip('\'"')
+                    if missing_key == 'retcode':
+                        log.debug('retcode missing from client return')
+                    else:
+                        log.debug(
+                            'Passing on saltutil error. Key \'%s\' missing '
+                            'from client return. This may be an error in '
+                            'the client.', missing_key
+                        )
                 # Keep track of the jid events to unsubscribe from later
                 open_jids.add(jinfo['jid'])
 
@@ -1405,6 +1556,15 @@ class LocalClient(object):
         '''
         log.trace('func get_cli_event_returns()')
 
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         if verbose:
             msg = 'Executing job with jid {0}'.format(jid)
             print(msg)
@@ -1424,6 +1584,7 @@ class LocalClient(object):
                                          expect_minions=(verbose or show_timeout),
                                          **kwargs
                                          ):
+            log.debug('return event: %s', ret)
             return_count = return_count + 1
             if progress:
                 for id_, min_ret in six.iteritems(ret):
@@ -1435,7 +1596,7 @@ class LocalClient(object):
                     if connected_minions is None:
                         connected_minions = salt.utils.minions.CkMinions(self.opts).connected_ids()
                     if self.opts['minion_data_cache'] \
-                            and salt.cache.Cache(self.opts).contains('minions/{0}'.format(id_), 'data') \
+                            and salt.cache.factory(self.opts).contains('minions/{0}'.format(id_), 'data') \
                             and connected_minions \
                             and id_ not in connected_minions:
 
@@ -1492,7 +1653,7 @@ class LocalClient(object):
                   tgt,
                   fun,
                   arg,
-                  expr_form,
+                  tgt_type,
                   ret,
                   jid,
                   timeout,
@@ -1500,7 +1661,7 @@ class LocalClient(object):
         '''
         Set up the payload_kwargs to be sent down to the master
         '''
-        if expr_form == 'nodegroup':
+        if tgt_type == 'nodegroup':
             if tgt not in self.opts['nodegroups']:
                 conf_file = self.opts.get(
                     'conf_file', 'the master config file'
@@ -1512,13 +1673,13 @@ class LocalClient(object):
                 )
             tgt = salt.utils.minions.nodegroup_comp(tgt,
                                                     self.opts['nodegroups'])
-            expr_form = 'compound'
+            tgt_type = 'compound'
 
         # Convert a range expression to a list of nodes and change expression
         # form to list
-        if expr_form == 'range' and HAS_RANGE:
+        if tgt_type == 'range' and HAS_RANGE:
             tgt = self._convert_range_to_list(tgt)
-            expr_form = 'list'
+            tgt_type = 'list'
 
         # If an external job cache is specified add it to the ret list
         if self.opts.get('ext_job_cache'):
@@ -1536,7 +1697,7 @@ class LocalClient(object):
                           'fun': fun,
                           'arg': arg,
                           'key': self.key,
-                          'tgt_type': expr_form,
+                          'tgt_type': tgt_type,
                           'ret': ret,
                           'jid': jid}
 
@@ -1560,7 +1721,7 @@ class LocalClient(object):
             tgt,
             fun,
             arg=(),
-            expr_form='glob',
+            tgt_type='glob',
             ret='',
             jid='',
             timeout=5,
@@ -1587,6 +1748,15 @@ class LocalClient(object):
             minions:
                 A set, the targets that the tgt passed should match.
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         # Make sure the publisher is running by checking the unix socket
         if (self.opts.get('ipc_mode', '') != 'tcp' and
                 not os.path.exists(os.path.join(self.opts['sock_dir'],
@@ -1601,7 +1771,7 @@ class LocalClient(object):
                 tgt,
                 fun,
                 arg,
-                expr_form,
+                tgt_type,
                 ret,
                 jid,
                 timeout,
@@ -1621,9 +1791,13 @@ class LocalClient(object):
             payload = channel.send(payload_kwargs, timeout=timeout)
         except SaltReqTimeoutError:
             raise SaltReqTimeoutError(
-                'Salt request timed out. The master is not responding. '
-                'If this error persists after verifying the master is up, '
-                'worker_threads may need to be increased.'
+                'Salt request timed out. The master is not responding. You '
+                'may need to run your command with `--async` in order to '
+                'bypass the congested event bus. With `--async`, the CLI tool '
+                'will print the job id (jid) and exit immediately without '
+                'listening for responses. You can then use '
+                '`salt-run jobs.lookup_jid` to look up the results of the job '
+                'in the job cache later.'
             )
 
         if not payload:
@@ -1654,7 +1828,7 @@ class LocalClient(object):
                   tgt,
                   fun,
                   arg=(),
-                  expr_form='glob',
+                  tgt_type='glob',
                   ret='',
                   jid='',
                   timeout=5,
@@ -1682,6 +1856,15 @@ class LocalClient(object):
             minions:
                 A set, the targets that the tgt passed should match.
         '''
+        if 'expr_form' in kwargs:
+            salt.utils.warn_until(
+                'Fluorine',
+                'The target type should be passed using the \'tgt_type\' '
+                'argument instead of \'expr_form\'. Support for using '
+                '\'expr_form\' will be removed in Salt Fluorine.'
+            )
+            tgt_type = kwargs.pop('expr_form')
+
         # Make sure the publisher is running by checking the unix socket
         if (self.opts.get('ipc_mode', '') != 'tcp' and
                 not os.path.exists(os.path.join(self.opts['sock_dir'],
@@ -1696,7 +1879,7 @@ class LocalClient(object):
                 tgt,
                 fun,
                 arg,
-                expr_form,
+                tgt_type,
                 ret,
                 jid,
                 timeout,
@@ -1717,9 +1900,13 @@ class LocalClient(object):
             payload = yield channel.send(payload_kwargs, timeout=timeout)
         except SaltReqTimeoutError:
             raise SaltReqTimeoutError(
-                'Salt request timed out. The master is not responding. '
-                'If this error persists after verifying the master is up, '
-                'worker_threads may need to be increased.'
+                'Salt request timed out. The master is not responding. You '
+                'may need to run your command with `--async` in order to '
+                'bypass the congested event bus. With `--async`, the CLI tool '
+                'will print the job id (jid) and exit immediately without '
+                'listening for responses. You can then use '
+                '`salt-run jobs.lookup_jid` to look up the results of the job '
+                'in the job cache later.'
             )
 
         if not payload:
