@@ -247,6 +247,20 @@ def _parse_domainname():
     return ''
 
 
+def _parse_searchdomain():
+    '''
+    Parse /etc/resolv.conf and return searchdomain
+    '''
+    contents = _read_file(_DEB_RESOLV_FILE)
+    pattern = r'search\s+(?P<search_domain>\S+)'
+    prog = re.compile(pattern)
+    for item in contents:
+        match = prog.match(item)
+        if match:
+            return match.group('search_domain')
+    return ''
+
+
 def _parse_hostname():
     '''
     Parse /etc/hostname and return hostname
@@ -275,9 +289,11 @@ def _parse_current_network_settings():
 
     hostname = _parse_hostname()
     domainname = _parse_domainname()
+    searchdomain = _parse_searchdomain()
 
     opts['hostname'] = hostname
     opts['domainname'] = domainname
+    opts['searchdomain'] = searchdomain
     return opts
 
 
@@ -392,6 +408,7 @@ def __space_delimited_list(value):
 
 SALT_ATTR_TO_DEBIAN_ATTR_MAP = {
     'dns': 'dns-nameservers',
+    'search': 'dns-search',
     'hwaddr': 'hwaddress',  # TODO: this limits bootp functionality
     'ipaddr': 'address',
 }
@@ -434,11 +451,13 @@ IPV4_ATTR_MAP = {
     'dstaddr':  __ipv4_quad,
     'local':  __ipv4_quad,
     'ttl':  __int,
+    # bond
+    'slaves': __anything,
     # ppp
     'provider': __anything,
     'unit':  __int,
     'options': __anything,
-    #
+    # resolvconf
     'dns-nameservers': __space_delimited_list,
     'dns-search': __space_delimited_list,
     #
@@ -455,12 +474,15 @@ IPV6_VALID_PROTO = ['auto', 'loopback', 'static', 'manual',
                     'dhcp', 'v4tunnel', '6to4']
 
 IPV6_ATTR_MAP = {
+    'proto': __within(IPV6_VALID_PROTO),
+    # ipv6 static & manual
     'address': __ipv6,
     'netmask': __ipv6_netmask,
     'broadcast': __ipv6,
     'gateway': __ipv6,  # supports a colon-delimited list
+    'hwaddress':  __mac,
+    'mtu':  __int,
     'scope': __within(['global', 'site', 'link', 'host'], dtype=str),
-    'proto': __within(IPV6_VALID_PROTO),
     # inet6 auto
     'privext': __within([0, 1, 2], dtype=int),
     'dhcp':  __within([0, 1], dtype=int),
@@ -471,6 +493,21 @@ IPV6_ATTR_MAP = {
     'preferred-lifetime':  __int,
     'dad-attempts': __int,  # 0 to disable
     'dad-interval': __float,
+    # bond
+    'slaves': __anything,
+    # tunnel
+    'mode':  __within(['gre', 'GRE', 'ipip', 'IPIP', '802.3ad'], dtype=str),
+    'endpoint': __ipv4_quad,
+    'local':  __ipv4_quad,
+    'ttl':  __int,
+    # resolvconf
+    'dns-nameservers': __space_delimited_list,
+    'dns-search': __space_delimited_list,
+    #
+    'vlan-raw-device': __anything,
+
+    'test': __anything,  # TODO
+    'enable_ipv6': __anything,  # TODO
 }
 
 
@@ -491,7 +528,7 @@ WIRELESS_ATTR_MAP = {
 
 ATTRMAPS = {
     'inet': [IPV4_ATTR_MAP, WIRELESS_ATTR_MAP],
-    'inet6': [IPV6_ATTR_MAP, IPV4_ATTR_MAP, WIRELESS_ATTR_MAP]
+    'inet6': [IPV6_ATTR_MAP, WIRELESS_ATTR_MAP]
 }
 
 
@@ -538,6 +575,8 @@ def _parse_interfaces(interface_files=None):
 
     for interface_file in interface_files:
         with salt.utils.fopen(interface_file) as interfaces:
+            # This ensures iface_dict exists, but does not ensure we're not reading a new interface.
+            iface_dict = {}
             for line in interfaces:
                 # Identify the clauses by the first word of each line.
                 # Go to the next line if the current line is a comment
@@ -882,17 +921,21 @@ def _parse_settings_bond_1(opts, iface, bond_def):
             _log_default_iface(iface, binding, bond_def[binding])
             bond.update({binding: bond_def[binding]})
 
-    if 'use_carrier' in opts:
-        if opts['use_carrier'] in _CONFIG_TRUE:
-            bond.update({'use_carrier': '1'})
-        elif opts['use_carrier'] in _CONFIG_FALSE:
-            bond.update({'use_carrier': '0'})
+    if 'primary' in opts:
+        bond.update({'primary': opts['primary']})
+
+    if not (__grains__['os'] == "Ubuntu" and __grains__['osrelease_info'][0] >= 16):
+        if 'use_carrier' in opts:
+            if opts['use_carrier'] in _CONFIG_TRUE:
+                bond.update({'use_carrier': '1'})
+            elif opts['use_carrier'] in _CONFIG_FALSE:
+                bond.update({'use_carrier': '0'})
+            else:
+                valid = _CONFIG_TRUE + _CONFIG_FALSE
+                _raise_error_iface(iface, 'use_carrier', valid)
         else:
-            valid = _CONFIG_TRUE + _CONFIG_FALSE
-            _raise_error_iface(iface, 'use_carrier', valid)
-    else:
-        _log_default_iface(iface, 'use_carrier', bond_def['use_carrier'])
-        bond.update({'use_carrier': bond_def['use_carrier']})
+            _log_default_iface(iface, 'use_carrier', bond_def['use_carrier'])
+            bond.update({'use_carrier': bond_def['use_carrier']})
 
     return bond
 
@@ -934,9 +977,6 @@ def _parse_settings_bond_2(opts, iface, bond_def):
         _log_default_iface(iface, 'arp_interval', bond_def['arp_interval'])
         bond.update({'arp_interval': bond_def['arp_interval']})
 
-    if 'primary' in opts:
-        bond.update({'primary': opts['primary']})
-
     if 'hashing-algorithm' in opts:
         valid = ['layer2', 'layer2+3', 'layer3+4']
         if opts['hashing-algorithm'] in valid:
@@ -1067,6 +1107,9 @@ def _parse_settings_bond_5(opts, iface, bond_def):
         _log_default_iface(iface, 'use_carrier', bond_def['use_carrier'])
         bond.update({'use_carrier': bond_def['use_carrier']})
 
+    if 'primary' in opts:
+        bond.update({'primary': opts['primary']})
+
     return bond
 
 
@@ -1103,6 +1146,9 @@ def _parse_settings_bond_6(opts, iface, bond_def):
         _log_default_iface(iface, 'use_carrier', bond_def['use_carrier'])
         bond.update({'use_carrier': bond_def['use_carrier']})
 
+    if 'primary' in opts:
+        bond.update({'primary': opts['primary']})
+
     return bond
 
 
@@ -1116,7 +1162,7 @@ def _parse_bridge_opts(opts, iface):
 
     if 'ports' in opts:
         if isinstance(opts['ports'], list):
-            opts['ports'] = ','.join(opts['ports'])
+            opts['ports'] = ' '.join(opts['ports'])
         config.update({'ports': opts['ports']})
 
     for opt in ['ageing', 'fd', 'gcint', 'hello', 'maxage']:
@@ -1190,6 +1236,7 @@ def _parse_settings_eth(opts, iface_type, enabled, iface):
     adapters[iface]['data'] = salt.utils.odict.OrderedDict()
     iface_data = adapters[iface]['data']
     iface_data['inet'] = salt.utils.odict.OrderedDict()
+    iface_data['inet6'] = salt.utils.odict.OrderedDict()
 
     if enabled:
         adapters[iface]['enabled'] = True
@@ -1197,7 +1244,29 @@ def _parse_settings_eth(opts, iface_type, enabled, iface):
     if opts.get('hotplug', False):
         adapters[iface]['hotplug'] = True
 
-    iface_data['inet']['addrfam'] = 'inet'
+    # Defaults assume IPv4 (inet) interfaces unless enable_ipv6=True
+    def_addrfam = 'inet'
+    dual_stack = False
+
+    # If enable_ipv6=True, then expet either IPv6-only or dual stack.
+    if 'enable_ipv6' in opts and opts['enable_ipv6']:
+        iface_data['inet6']['addrfam'] = 'inet6'
+        iface_data['inet6']['netmask'] = '64'  # defaults to 64
+        def_addrfam = 'inet6'
+
+        if 'iface_type' in opts and opts['iface_type'] == 'vlan':
+            iface_data['inet6']['vlan_raw_device'] = (
+                re.sub(r'\.\d*', '', iface))
+
+        if 'ipaddr' in opts and 'ipv6ipaddr' in opts:
+            # If both 'ipaddr' and 'ipv6ipaddr' are present; expect dual stack
+            iface_data['inet']['addrfam'] = 'inet'
+            def_addrfam = 'inet'
+            dual_stack = True
+
+    else:
+        # If enable_ipv6=False|None, IPv6 settings should not be set.
+        iface_data['inet']['addrfam'] = 'inet'
 
     if iface_type not in ['bridge']:
         tmp_ethtool = _parse_ethtool_opts(opts, iface)
@@ -1206,54 +1275,53 @@ def _parse_settings_eth(opts, iface_type, enabled, iface):
             for item in tmp_ethtool:
                 ethtool[_ETHTOOL_CONFIG_OPTS[item]] = tmp_ethtool[item]
 
-            iface_data['inet']['ethtool'] = ethtool
+            iface_data[def_addrfam]['ethtool'] = ethtool
             # return a list of sorted keys to ensure consistent order
-            iface_data['inet']['ethtool_keys'] = sorted(ethtool)
+            iface_data[def_addrfam]['ethtool_keys'] = sorted(ethtool)
 
     if iface_type == 'bridge':
         bridging = _parse_bridge_opts(opts, iface)
         if bridging:
             opts.pop('mode', None)
-            iface_data['inet']['bridging'] = bridging
-            iface_data['inet']['bridging_keys'] = sorted(bridging)
+            iface_data[def_addrfam]['bridging'] = bridging
+            iface_data[def_addrfam]['bridging_keys'] = sorted(bridging)
+            iface_data[def_addrfam]['addrfam'] = def_addrfam
 
     elif iface_type == 'bond':
         bonding = _parse_settings_bond(opts, iface)
         if bonding:
             opts.pop('mode', None)
-            iface_data['inet']['bonding'] = bonding
-            iface_data['inet']['bonding']['slaves'] = opts['slaves']
-            iface_data['inet']['bonding_keys'] = sorted(bonding)
+            iface_data[def_addrfam]['bonding'] = bonding
+            iface_data[def_addrfam]['bonding']['slaves'] = opts['slaves']
+            iface_data[def_addrfam]['bonding_keys'] = sorted(bonding)
+            iface_data[def_addrfam]['addrfam'] = def_addrfam
 
     elif iface_type == 'slave':
         adapters[iface]['master'] = opts['master']
 
+        opts['proto'] = 'manual'
+        iface_data[def_addrfam]['master'] = adapters[iface]['master']
+        iface_data[def_addrfam]['addrfam'] = def_addrfam
+
     elif iface_type == 'vlan':
-        iface_data['inet']['vlan_raw_device'] = re.sub(r'\.\d*', '', iface)
+        iface_data[def_addrfam]['vlan_raw_device'] = re.sub(r'\.\d*', '', iface)
+        iface_data[def_addrfam]['addrfam'] = def_addrfam
 
     elif iface_type == 'pppoe':
         tmp_ethtool = _parse_ethtool_pppoe_opts(opts, iface)
         if tmp_ethtool:
             for item in tmp_ethtool:
-                adapters[iface]['data']['inet'][_DEB_CONFIG_PPPOE_OPTS[item]] = tmp_ethtool[item]
-
-    if 'enable_ipv6' in opts and opts['enable_ipv6']:  # TODO: check yes/no
-        # iface_data.append({})
-        iface_data['inet6'] = {}
-        iface_data['inet6']['addrfam'] = 'inet6'
-        iface_data['inet6']['netmask'] = '64'  # defaults to 64
-
-        if 'iface_type' in opts and opts['iface_type'] == 'vlan':
-            iface_data['inet6']['vlan_raw_device'] = (
-                re.sub(r'\.\d*', '', iface))
+                adapters[iface]['data'][def_addrfam][_DEB_CONFIG_PPPOE_OPTS[item]] = tmp_ethtool[item]
+        iface_data[def_addrfam]['addrfam'] = def_addrfam
 
     for opt in opts:
+        # trim leading "ipv6" from option
         if opt.startswith('ipv6'):
             optname = opt[4:]  # trim off the ipv6
-            addrfam = 'inet6'
+            v6only = True
         else:
             optname = opt
-            addrfam = 'inet'
+            v6only = False
 
         _optname = SALT_ATTR_TO_DEBIAN_ATTR_MAP.get(optname, optname)
         if _attrmaps_contain_attr(_optname):
@@ -1263,25 +1331,71 @@ def _parse_settings_eth(opts, iface_type, enabled, iface):
             if optname == 'proto' and valuestr == 'none':
                 valuestr = 'static'
 
-            (valid, value, errmsg) = _validate_interface_option(
-                _optname, valuestr, addrfam=addrfam)
+            # If option is v6-only, don't validate against inet and always set value
+            if v6only:
+                (valid, value, errmsg) = _validate_interface_option(
+                   _optname, valuestr, addrfam='inet6')
 
-            if not valid:
-                _raise_error_iface(
-                    iface,
-                    '\'{0}\' \'{1}\''.format(opt, valuestr),
-                    [errmsg]
-                )
+                if not valid:
+                    _raise_error_iface(iface, '\'{0}\' \'{1}\''.format(opt, valuestr), [errmsg])
 
-            # replace dashes with underscores for jinja
-            _optname = _optname.replace('-', '_')
-            iface_data[addrfam][_optname] = value
+                # replace dashes with underscores for jinja
+                _optname = _optname.replace('-', '_')
+                iface_data['inet6'][_optname] = value
+
+            # Else, if it's a dual stack, the option may belong in both; apply v4 opt as v6 default
+            elif dual_stack:
+                valid_once = False
+                errmsg = None
+                for addrfam in ['inet', 'inet6']:
+                    (valid, value, errmsg) = _validate_interface_option(
+                       _optname, valuestr, addrfam=addrfam)
+
+                    if valid:
+                        valid_once = True
+                        # replace dashes with underscores for jinja
+                        _optname = _optname.replace('-', '_')
+                        # if a v6-only version of this option was set; don't override
+                        # otherwise, if dual stack, use the v4 version as a default value for v6
+                        # allows overriding with =None
+                        if addrfam == 'inet' or _optname not in iface_data['inet6']:
+                            iface_data[addrfam][_optname] = value
+
+                if not valid_once:
+                    _raise_error_iface(
+                        iface,
+                        '\'{0}\' \'{1}\''.format(opt, valuestr),
+                        [errmsg]
+                    )
+
+            # Else, it goes in the default(only) addrfam
+            # Not assuming v4 allows a v6 block to be created without lots of "ipv6" prefixes
+            else:
+                (valid, value, errmsg) = _validate_interface_option(
+                   _optname, valuestr, addrfam=def_addrfam)
+
+                if not valid:
+                    _raise_error_iface(
+                        iface,
+                        '\'{0}\' \'{1}\''.format(opt, valuestr),
+                        [errmsg]
+                    )
+
+                # replace dashes with underscores for jinja
+                _optname = _optname.replace('-', '_')
+                iface_data[def_addrfam][_optname] = value
 
     for opt in ['up_cmds', 'pre_up_cmds', 'post_up_cmds',
                 'down_cmds', 'pre_down_cmds', 'post_down_cmds']:
         if opt in opts:
             iface_data['inet'][opt] = opts[opt]
 
+    for addrfam in ['inet', 'inet6']:
+        if 'addrfam' in iface_data[addrfam] and iface_data[addrfam]['addrfam'] == addrfam:
+            pass
+        else:
+            iface_data.pop(addrfam)
+
     return adapters
 
 
@@ -1342,6 +1456,9 @@ def _parse_network_settings(opts, current):
     else:
         _raise_error_network('hostname', ['server1.example.com'])
 
+    if 'search' in opts:
+        result['search'] = opts['search']
+
     return result
 
 
@@ -1456,12 +1573,6 @@ def _write_file_ifaces(iface, data, **settings):
 
     ifcfg = ''
     for adapter in adapters:
-        if 'type' in adapters[adapter] and adapters[adapter]['type'] == 'slave':
-            # Override values so the interfaces file is correct
-            adapters[adapter]['data']['inet']['addrfam'] = 'inet'
-            adapters[adapter]['data']['inet']['proto'] = 'manual'
-            adapters[adapter]['data']['inet']['master'] = adapters[adapter]['master']
-
         if 'type' in adapters[adapter] and adapters[adapter]['type'] == 'source':
             tmp = source_template.render({'name': adapter, 'data': adapters[adapter]})
         else:
@@ -1470,13 +1581,13 @@ def _write_file_ifaces(iface, data, **settings):
         if adapter == iface:
             saved_ifcfg = tmp
 
-    _SEPERATE_FILE = False
+    _SEPARATE_FILE = False
     if 'filename' in settings:
         if not settings['filename'].startswith('/'):
             filename = '{0}/{1}'.format(_DEB_NETWORK_DIR, settings['filename'])
         else:
             filename = settings['filename']
-        _SEPERATE_FILE = True
+        _SEPARATE_FILE = True
     else:
         if 'filename' in adapters[adapter]['data']:
             filename = adapters[adapter]['data']
@@ -1489,7 +1600,7 @@ def _write_file_ifaces(iface, data, **settings):
         log.error(msg)
         raise AttributeError(msg)
     with salt.utils.flopen(filename, 'w') as fout:
-        if _SEPERATE_FILE:
+        if _SEPARATE_FILE:
             fout.write(saved_ifcfg)
         else:
             fout.write(ifcfg)
@@ -1921,6 +2032,7 @@ def build_network_settings(**settings):
     opts['hostname'] = sline[0]
     hostname = '{0}\n' . format(opts['hostname'])
     current_domainname = current_network_settings['domainname']
+    current_searchdomain = current_network_settings['searchdomain']
 
     # Only write the hostname if it has changed
     if not opts['hostname'] == current_network_settings['hostname']:
@@ -1942,18 +2054,38 @@ def build_network_settings(**settings):
         domainname = current_domainname
         opts['domainname'] = domainname
 
+    new_search = False
+    if 'search' in opts:
+        new_searchdomain = opts['search']
+        if new_searchdomain != current_searchdomain:
+            searchdomain = new_searchdomain
+            opts['searchdomain'] = new_searchdomain
+            new_search = True
+        else:
+            searchdomain = current_searchdomain
+            opts['searchdomain'] = searchdomain
+    else:
+        searchdomain = current_searchdomain
+        opts['searchdomain'] = searchdomain
+
     # If the domain changes, then we should write the resolv.conf file.
-    if new_domain:
+    if new_domain or new_search:
         # Look for existing domain line and update if necessary
         contents = _parse_resolve()
-        prog = re.compile(r'domain\s+(?P<domain_name>\S+)')
+        domain_prog = re.compile(r'domain\s+(?P<domain_name>\S+)')
+        search_prog = re.compile(r'search\s+(?P<search_domain>\S+)')
         new_contents = []
         found_domain = False
+        found_search = False
         for item in contents:
-            match = prog.match(item)
-            if match:
+            domain_match = domain_prog.match(item)
+            search_match = search_prog.match(item)
+            if domain_match:
                 new_contents.append('domain {0}\n' . format(domainname))
                 found_domain = True
+            elif search_match:
+                new_contents.append('search {0}\n' . format(searchdomain))
+                found_search = True
             else:
                 new_contents.append(item)
 
@@ -1961,6 +2093,15 @@ def build_network_settings(**settings):
         # with the new domainname
         if not found_domain:
             new_contents.insert(0, 'domain {0}\n' . format(domainname))
+
+        # A search line didn't exist so we'll add one in
+        # with the new search domain
+        if not found_search:
+            if new_contents[0].startswith('domain'):
+                new_contents.insert(1, 'search {0}\n' . format(searchdomain))
+            else:
+                new_contents.insert(0, 'search {0}\n' . format(searchdomain))
+
         new_resolv = ''.join(new_contents)
 
         # Write /etc/resolv.conf
