@@ -3,7 +3,12 @@
 Module for managing windows systems.
 
 :depends:
+    - pythoncom
+    - pywintypes
+    - win32api
+    - win32con
     - win32net
+    - wmi
 
 Support for reboot, shutdown, etc
 '''
@@ -12,6 +17,7 @@ from __future__ import absolute_import
 # Import python libs
 import logging
 import time
+import ctypes
 from datetime import datetime
 
 # Import 3rd Party Libs
@@ -22,7 +28,6 @@ try:
     import win32api
     import win32con
     import pywintypes
-    import ctypes
     from ctypes import windll
     HAS_WIN32NET_MODS = True
 except ImportError:
@@ -42,11 +47,15 @@ __virtualname__ = 'system'
 
 def __virtual__():
     '''
-    Set the system module of the kernel is Windows
+    Only works on Windows Systems with Win32 Modules
     '''
-    if HAS_WIN32NET_MODS and salt.utils.is_windows():
-        return __virtualname__
-    return (False, "Module win_system: module only works on Windows systems")
+    if not salt.utils.is_windows():
+        return False, 'Module win_system: Requires Windows'
+
+    if not HAS_WIN32NET_MODS:
+        return False, 'Module win_system: Missing win32 modules'
+
+    return __virtualname__
 
 
 def _convert_minutes_seconds(timeout, in_seconds=False):
@@ -65,24 +74,32 @@ def _convert_date_time_string(dt_string):
     return dt_obj.strftime('%Y-%m-%d %H:%M:%S')
 
 
+def _to_unicode(instr):
+    '''
+    Converts from current users character encoding to unicode.
+    When instr has a value of None, the return value of the function
+    will also be None.
+    '''
+    if instr is None or isinstance(instr, six.text_type):
+        return instr
+    else:
+        return six.text_type(instr, 'utf8')
+
+
 def halt(timeout=5, in_seconds=False):
     '''
     Halt a running system.
 
-    :param int timeout:
-        Number of seconds before halting the system.
-        Default is 5 seconds.
+    Args:
+        timeout (int): Number of seconds before halting the system. Default is
+        5 seconds.
 
-    :return: True is successful.
-    :rtype: bool
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
 
-    timeout
-        The wait time before the system will be shutdown.
+            .. versionadded:: 2015.8.0
 
-    in_seconds
-        Whether to treat timeout as seconds or minutes.
-
-        .. versionadded:: 2015.8.0
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -117,20 +134,15 @@ def poweroff(timeout=5, in_seconds=False):
     '''
     Power off a running system.
 
-    :param int timeout:
-        Number of seconds before powering off the system.
-        Default is 5 seconds.
+    Args:
+        timeout (int): Number of seconds before powering off the system. Default
+            is 5 seconds.
 
-    :return: True if successful
-    :rtype: bool
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
+            .. versionadded:: 2015.8.0
 
-    timeout
-        The wait time before the system will be shutdown.
-
-    in_seconds
-        Whether to treat timeout as seconds or minutes.
-
-        .. versionadded:: 2015.8.0
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -146,35 +158,30 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False,  # pylint: disabl
     '''
     Reboot a running system.
 
-    :param int timeout:
-        Number of minutes/seconds before rebooting the system. Minutes vs
-        seconds depends on the value of ``in_seconds``.
-        Default is 5 minutes.
+    Args:
+        timeout (int): Number of minutes/seconds before rebooting the system.
+            Minutes vs seconds depends on the value of ``in_seconds``. Default
+            is 5 minutes.
 
-    :param bool in_seconds:
-        Whether to treat timeout as seconds or minutes.
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
 
-        .. versionadded:: 2015.8.0
+            .. versionadded:: 2015.8.0
 
-    :param bool wait_for_reboot:
+        wait_for_reboot (bool): Sleeps for timeout + 30 seconds after reboot has
+            been initiated. This may be useful for use in a highstate if a
+            reboot should be performed and the return data of the highstate is
+            not required. If return data is required, consider using the reboot
+            state instead of this module.
 
-        Sleeps for timeout + 30 seconds after reboot has been initiated.
-        This is useful for use in a highstate for example where
-        you have many states that could be ran after this one. Which you don't want
-        to start until after the restart i.e You could end up with a half finished state.
+            .. versionadded:: 2015.8.0
 
-        .. versionadded:: 2015.8.0
+        only_on_pending_reboot (bool): If this is set to True, then the reboot
+            will only proceed if the system reports a pending reboot. To
+            optionally reboot in a highstate, consider using the reboot state
+            instead of this module.
 
-    :param bool only_on_pending_reboot:
-
-        If this is set to True, then then the reboot will only proceed
-        if the system reports a pending reboot. Setting this paramater to
-        True could be useful when calling this function from a final housekeeping
-        state intended to be executed
-        at the end of a state run (using *order: last*).
-
-    :return: True if successful (a reboot will occur)
-    :rtype: bool
+    Returns:
+        bool: True if successful (a reboot will occur), otherwise False
 
     CLI Example:
 
@@ -195,7 +202,6 @@ def reboot(timeout=5, in_seconds=False, wait_for_reboot=False,  # pylint: disabl
               - name: system.reboot
               - only_on_pending_reboot: True
               - order: last
-
     '''
 
     ret = shutdown(timeout=timeout, reboot=True, in_seconds=in_seconds,
@@ -213,46 +219,45 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint:
     '''
     Shutdown a running system.
 
-    :param str message:
-        A message to display to the user before shutting down.
+    Args:
+        message (str): A message to display to the user before shutting down.
 
-    :param int timeout:
-        The length of time that the shutdown dialog box should be displayed, in
-        seconds. While this dialog box is displayed, the shutdown can be stopped
-        by the shutdown_abort function.
+        timeout (int): The length of time that the shutdown dialog box should be
+            displayed, in seconds. While this dialog box is displayed, the
+            shutdown can be stopped by the shutdown_abort function.
 
-        If timeout is not zero, InitiateSystemShutdown displays a dialog box on
-        the specified computer. The dialog box displays the name of the user
-        who called the function, displays the message specified by the
-        lpMessage parameter, and prompts the user to log off. The dialog box
-        beeps when it is created and remains on top of other windows in the
-        system. The dialog box can be moved but not closed. A timer counts down
-        the remaining time before a forced shutdown.
+            If timeout is not zero, InitiateSystemShutdown displays a dialog box
+            on the specified computer. The dialog box displays the name of the
+            user who called the function, displays the message specified by the
+            lpMessage parameter, and prompts the user to log off. The dialog box
+            beeps when it is created and remains on top of other windows in the
+            system. The dialog box can be moved but not closed. A timer counts
+            down the remaining time before a forced shutdown.
 
-        If timeout is zero, the computer shuts down without displaying the
-        dialog box, and the shutdown cannot be stopped by shutdown_abort.
+            If timeout is zero, the computer shuts down without displaying the
+            dialog box, and the shutdown cannot be stopped by shutdown_abort.
 
-        Default is 5 minutes
+            Default is 5 minutes
 
-    :param bool in_seconds:
-        Whether to treat timeout as seconds or minutes.
+        in_seconds (bool): Whether to treat timeout as seconds or minutes.
 
-        .. versionadded:: 2015.8.0
+            .. versionadded:: 2015.8.0
 
-    :param bool force_close:
-        True to force close all open applications. False displays a dialog box
-        instructing the user to close the applications.
+        force_close (bool): True to force close all open applications. False
+            displays a dialog box instructing the user to close the
+            applications.
 
-    :param bool reboot:
-        True restarts the computer immediately after shutdown.
-        False caches to disk and safely powers down the system.
+        reboot (bool): True restarts the computer immediately after shutdown.
+            False caches to disk and safely powers down the system.
 
-    :param bool only_on_pending_reboot:
-        If this is set to True, then then shutdown will only proceed
-        if the system reports a pending reboot.
+        only_on_pending_reboot (bool): If this is set to True, then the shutdown
+            will only proceed if the system reports a pending reboot. To
+            optionally shutdown in a highstate, consider using the shutdown
+            state instead of this module.
 
-    :return: True if successful (a shutdown or reboot will occur)
-    :rtype: bool
+    Returns:
+        bool: True if successful (a shutdown or reboot will occur), otherwise
+            False
 
     CLI Example:
 
@@ -260,6 +265,9 @@ def shutdown(message=None, timeout=5, force_close=True, reboot=False,  # pylint:
 
         salt '*' system.shutdown 5
     '''
+    if six.PY2:
+        message = _to_unicode(message)
+
     timeout = _convert_minutes_seconds(timeout, in_seconds)
 
     if only_on_pending_reboot and not get_pending_reboot():
@@ -284,8 +292,8 @@ def shutdown_hard():
     '''
     Shutdown a running system with no timeout or warning.
 
-    :return: True if successful
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -299,10 +307,11 @@ def shutdown_hard():
 def shutdown_abort():
     '''
     Abort a shutdown. Only available while the dialog box is being
-    displayed to the user. Once the shutdown has initiated, it cannot be aborted
+    displayed to the user. Once the shutdown has initiated, it cannot be
+    aborted.
 
-    :return: True if successful
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -326,8 +335,8 @@ def lock():
     '''
     Lock the workstation.
 
-    :return: True if successful
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -342,12 +351,13 @@ def set_computer_name(name):
     '''
     Set the Windows computer name
 
-    :param str name:
-        The new name to give the computer. Requires a reboot to take effect.
+    Args:
+        name (str): The new name to give the computer. Requires a reboot to take
+            effect.
 
-    :return:
-        Returns a dictionary containing the old and new names if successful.
-        False if not.
+    Returns:
+        dict: Returns a dictionary containing the old and new names if
+            successful. False if not.
 
     CLI Example:
 
@@ -355,16 +365,17 @@ def set_computer_name(name):
 
         salt 'minion-id' system.set_computer_name 'DavesComputer'
     '''
-    if name and six.PY2:
-        name = name.decode('utf-8')
+    if six.PY2:
+        name = _to_unicode(name)
 
-    if windll.kernel32.SetComputerNameExW(win32con.ComputerNamePhysicalDnsHostname,
-                                          name):
+    if windll.kernel32.SetComputerNameExW(
+            win32con.ComputerNamePhysicalDnsHostname, name):
         ret = {'Computer Name': {'Current': get_computer_name()}}
         pending = get_pending_computer_name()
         if pending not in (None, False):
             ret['Computer Name']['Pending'] = pending
         return ret
+
     return False
 
 
@@ -376,9 +387,8 @@ def get_pending_computer_name():
     retrieving the pending computer name, ``False`` will be returned, and an
     error message will be logged to the minion log.
 
-    :return:
-        Returns the pending name if pending restart. Returns none if not pending
-        restart.
+    Returns:
+        str: The pending name if restart is pending, otherwise returns None.
 
     CLI Example:
 
@@ -400,8 +410,8 @@ def get_computer_name():
     '''
     Get the Windows computer name
 
-    :return:
-        Returns the computer name if found. Otherwise returns False
+    Returns:
+        str: Returns the computer name if found. Otherwise returns False
 
     CLI Example:
 
@@ -417,10 +427,11 @@ def set_computer_desc(desc=None):
     '''
     Set the Windows computer description
 
-    :param str desc:
-        The computer description
+    Args:
+        desc (str): The computer description
 
-    :return: False if it fails. Description if successful.
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -428,6 +439,9 @@ def set_computer_desc(desc=None):
 
         salt 'minion-id' system.set_computer_desc 'This computer belongs to Dave!'
     '''
+    if six.PY2:
+        desc = _to_unicode(desc)
+
     # Make sure the system exists
     # Return an object containing current information array for the computer
     system_info = win32net.NetServerGetInfo(None, 101)
@@ -435,8 +449,7 @@ def set_computer_desc(desc=None):
     # If desc is passed, decode it for unicode
     if desc is None:
         return False
-    if not isinstance(desc, str):
-        desc = desc.decode('utf-8')
+
     system_info['comment'] = desc
 
     # Apply new settings
@@ -460,10 +473,9 @@ def get_system_info():
     '''
     Get system information.
 
-    :return:
-        Returns a Dictionary containing information about the system to include
-        name, description, version, etc...
-    :rtype: dict
+    Returns:
+        dict: Returns a Dictionary containing information about the system to
+            include name, description, version, etc...
 
     CLI Example:
 
@@ -514,8 +526,9 @@ def get_system_info():
 def get_computer_desc():
     '''
     Get the Windows computer description
-    :return:
-        Returns the computer description if found. Otherwise returns False
+
+    Returns:
+        str: The computer description if found, otherwise False
 
     CLI Example:
 
@@ -536,8 +549,8 @@ def get_hostname():
 
     Get the hostname of the windows minion
 
-    :return:
-        Returns the hostname of the windows minion
+    Returns:
+        str: The hostname of the windows minion
 
     CLI Example:
 
@@ -557,8 +570,11 @@ def set_hostname(hostname):
     Set the hostname of the windows minion, requires a restart before this
     will be updated.
 
-    :param str hostname:
-        The hostname to set
+    Args:
+        hostname (str): The hostname to set
+
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -573,33 +589,6 @@ def set_hostname(hostname):
     return "successful" in ret
 
 
-def _lookup_error(number):
-    '''
-    Lookup the error based on the passed number
-    .. versionadded:: 2015.5.7
-    .. versionadded:: 2015.8.2
-
-    :param int number: Number code to lookup
-
-    :return: The text that corresponds to the error number
-    :rtype: str
-    '''
-    return_values = {
-        2:    'Invalid OU or specifying OU is not supported',
-        5:    'Access is denied',
-        53:   'The network path was not found',
-        87:   'The parameter is incorrect',
-        110:  'The system cannot open the specified object',
-        1323: 'Unable to update the password',
-        1326: 'Logon failure: unknown username or bad password',
-        1355: 'The specified domain either does not exist or could not be contacted',
-        2224: 'The account already exists',
-        2691: 'The machine is already joined to the domain',
-        2692: 'The machine is not currently joined to a domain',
-    }
-    return return_values[number]
-
-
 def join_domain(domain,
                 username=None,
                 password=None,
@@ -609,35 +598,31 @@ def join_domain(domain,
     '''
     Join a computer to an Active Directory domain. Requires reboot.
 
-    :param str domain:
-        The domain to which the computer should be joined, e.g.
-        ``example.com``
+    Args:
+        domain (str): The domain to which the computer should be joined, e.g.
+            ``example.com``
 
-    :param str username:
-        Username of an account which is authorized to join computers to the
-        specified domain. Need to be either fully qualified like
-        ``user@domain.tld`` or simply ``user``
+        username (str): Username of an account which is authorized to join
+            computers to the specified domain. Need to be either fully qualified
+            like ``user@domain.tld`` or simply ``user``
 
-    :param str password:
-        Password of the specified user
+        password (str): Password of the specified user
 
-    :param str account_ou:
-        The DN of the OU below which the account for this computer should be
-        created when joining the domain, e.g.
-        ``ou=computers,ou=departm_432,dc=my-company,dc=com``
+        account_ou (str): The DN of the OU below which the account for this
+            computer should be created when joining the domain, e.g.
+            ``ou=computers,ou=departm_432,dc=my-company,dc=com``
 
-    :param bool account_exists:
-        If set to ``True`` the computer will only join the domain if the account
-        already exists. If set to ``False`` the computer account will be created
-        if it does not exist, otherwise it will use the existing account.
-        Default is False
+        account_exists (bool): If set to ``True`` the computer will only join
+            the domain if the account already exists. If set to ``False`` the
+            computer account will be created if it does not exist, otherwise it
+            will use the existing account. Default is False.
 
-    :param bool restart: Restarts the computer after a successful join
+        restart (bool): Restarts the computer after a successful join
 
-        .. versionadded:: 2015.8.2/2015.5.7
+            .. versionadded:: 2015.8.2/2015.5.7
 
-    :returns: Returns a dictionary if successful. False if unsuccessful.
-    :rtype: dict, bool
+    Returns:
+        dict: Dictionary if successful, otherwise False
 
     CLI Example:
 
@@ -648,6 +633,12 @@ def join_domain(domain,
                          account_ou='ou=clients,ou=org,dc=domain,dc=tld' \\
                          account_exists=False, restart=True
     '''
+    if six.PY2:
+        domain = _to_unicode(domain)
+        username = _to_unicode(username)
+        password = _to_unicode(password)
+        account_ou = _to_unicode(account_ou)
+
     status = get_domain_workgroup()
     if 'Domain' in status:
         if status['Domain'] == domain:
@@ -694,7 +685,7 @@ def join_domain(domain,
             ret['Restart'] = reboot()
         return ret
 
-    log.error(_lookup_error(err[0]))
+    log.error(win32api.FormatMessage(err[0]).rstrip())
     return False
 
 
@@ -707,32 +698,31 @@ def unjoin_domain(username=None,
     r'''
     Unjoin a computer from an Active Directory Domain. Requires restart.
 
-    :param username:
-        Username of an account which is authorized to manage computer accounts
-        on the domain. Need to be fully qualified like ``user@domain.tld`` or
-        ``domain.tld\user``. If domain not specified, the passed domain will be
-        used. If computer account doesn't need to be disabled, can be None.
+    Args:
+        username (str): Username of an account which is authorized to manage
+            computer accounts on the domain. Need to be fully qualified like
+            ``user@domain.tld`` or ``domain.tld\user``. If domain not specified,
+            the passed domain will be used. If computer account doesn't need to
+            be disabled, can be None.
 
-    :param str password:
-        Password of the specified user
+        password (str): Password of the specified user
 
-    :param str domain: The domain from which to unjoin the computer. Can be None
+        domain (str): The domain from which to unjoin the computer. Can be None.
 
-    :param str workgroup: The workgroup to join the computer to. Default is
-    ``WORKGROUP``
+        workgroup (str): The workgroup to join the computer to. Default is
+            ``WORKGROUP``
 
-        .. versionadded:: 2015.8.2/2015.5.7
+            .. versionadded:: 2015.8.2/2015.5.7
 
-    :param bool disable:
-        Disable the computer account in Active Directory. True to disable.
-        Default is False
+        disable (bool): Disable the computer account in Active Directory. True
+            to disable. Default is False
 
-    :param bool restart: Restart the computer after successful unjoin
+        restart (bool): Restart the computer after successful unjoin
 
-        .. versionadded:: 2015.8.2/2015.5.7
+            .. versionadded:: 2015.8.2/2015.5.7
 
-    :returns: Returns a dictionary if successful. False if unsuccessful.
-    :rtype: dict, bool
+    Returns:
+        dict: Dictionary if successful, otherwise False
 
     CLI Example:
 
@@ -744,6 +734,11 @@ def unjoin_domain(username=None,
                          password='unjoinpassword' disable=True \\
                          restart=True
     '''
+    if six.PY2:
+        username = _to_unicode(username)
+        password = _to_unicode(password)
+        domain = _to_unicode(domain)
+
     status = get_domain_workgroup()
     if 'Workgroup' in status:
         if status['Workgroup'] == workgroup:
@@ -758,7 +753,7 @@ def unjoin_domain(username=None,
     if username and password is None:
         return 'Must specify a password if you pass a username'
 
-    NETSETUP_ACCT_DELETE = 0x2  # pylint: disable=invalid-name
+    NETSETUP_ACCT_DELETE = 0x4  # pylint: disable=invalid-name
 
     unjoin_options = 0x0
     if disable:
@@ -783,11 +778,11 @@ def unjoin_domain(username=None,
 
             return ret
         else:
-            log.error(_lookup_error(err[0]))
+            log.error(win32api.FormatMessage(err[0]).rstrip())
             log.error('Failed to join the computer to {0}'.format(workgroup))
             return False
     else:
-        log.error(_lookup_error(err[0]))
+        log.error(win32api.FormatMessage(err[0]).rstrip())
         log.error('Failed to unjoin computer from {0}'.format(status['Domain']))
         return False
 
@@ -799,15 +794,14 @@ def get_domain_workgroup():
     .. versionadded:: 2015.5.7
     .. versionadded:: 2015.8.2
 
-    :return: The name of the domain or workgroup
-    :rtype: str
+    Returns:
+        str: The name of the domain or workgroup
 
     CLI Example:
 
     .. code-block:: bash
 
         salt 'minion-id' system.get_domain_workgroup
-
     '''
     pythoncom.CoInitialize()
     conn = wmi.WMI()
@@ -822,11 +816,13 @@ def _try_parse_datetime(time_str, fmts):
     '''
     Attempts to parse the input time_str as a date.
 
-    :param str time_str: A string representing the time
-    :param list fmts: A list of date format strings
+    Args:
+        time_str (str): A string representing the time
 
-    :return: Returns a datetime object if parsed properly. Otherwise None
-    :rtype datetime
+        fmts (list): A list of date format strings
+
+    Returns:
+        datetime: A datetime object if parsed properly, otherwise None
     '''
     result = None
     for fmt in fmts:
@@ -842,8 +838,8 @@ def get_system_time():
     '''
     Get the system time.
 
-    :return: Returns the system time in HH:MM:SS AM/PM format.
-    :rtype: str
+    Returns:
+        str: Returns the system time in HH:MM:SS AM/PM format.
 
     CLI Example:
 
@@ -868,15 +864,16 @@ def set_system_time(newtime):
     '''
     Set the system time.
 
-    :param str newtime:
-        The time to set. Can be any of the following formats.
-        - HH:MM:SS AM/PM
-        - HH:MM AM/PM
-        - HH:MM:SS (24 hour)
-        - HH:MM (24 hour)
+    Args:
+        newtime (str): The time to set. Can be any of the following formats.
 
-    :return: Returns True if successful. Otherwise False.
-    :rtype: bool
+            - HH:MM:SS AM/PM
+            - HH:MM AM/PM
+            - HH:MM:SS (24 hour)
+            - HH:MM (24 hour)
+
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -908,15 +905,21 @@ def set_system_date_time(years=None,
     element will be used. For example, if you don't pass the year, the current
     system year will be used. (Used by set_system_date and set_system_time)
 
-    :param int years: Years digit, ie: 2015
-    :param int months: Months digit: 1 - 12
-    :param int days: Days digit: 1 - 31
-    :param int hours: Hours digit: 0 - 23
-    :param int minutes: Minutes digit: 0 - 59
-    :param int seconds: Seconds digit: 0 - 59
+    Args:
+        years (int): Years digit, ie: 2015
 
-    :return: True if successful. Otherwise False.
-    :rtype: bool
+        months (int): Months digit: 1 - 12
+
+        days (int): Days digit: 1 - 31
+
+        hours (int): Hours digit: 0 - 23
+
+        minutes (int): Minutes digit: 0 - 59
+
+        seconds (int): Seconds digit: 0 - 59
+
+    Returns:
+        bool: True if successful, otherwise False.
 
     CLI Example:
 
@@ -981,8 +984,8 @@ def get_system_date():
     '''
     Get the Windows system date
 
-    :return: Returns the system date.
-    :rtype: str
+    Returns:
+        str: The system date
 
     CLI Example:
 
@@ -998,14 +1001,15 @@ def set_system_date(newdate):
     '''
     Set the Windows system date. Use <mm-dd-yy> format for the date.
 
-    :param str newdate:
-        The date to set. Can be any of the following formats
-        - YYYY-MM-DD
-        - MM-DD-YYYY
-        - MM-DD-YY
-        - MM/DD/YYYY
-        - MM/DD/YY
-        - YYYY/MM/DD
+    Args:
+        newdate (str): The date to set. Can be any of the following formats:
+
+            - YYYY-MM-DD
+            - MM-DD-YYYY
+            - MM-DD-YY
+            - MM/DD/YYYY
+            - MM/DD/YY
+            - YYYY/MM/DD
 
     CLI Example:
 
@@ -1030,8 +1034,8 @@ def start_time_service():
     '''
     Start the Windows time service
 
-    :return: True if successful. Otherwise False
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False.
 
     CLI Example:
 
@@ -1046,8 +1050,8 @@ def stop_time_service():
     '''
     Stop the Windows time service
 
-    :return: True if successful. Otherwise False
-    :rtype: bool
+    Returns:
+        bool: True if successful, otherwise False
 
     CLI Example:
 
@@ -1060,13 +1064,14 @@ def stop_time_service():
 
 def get_pending_component_servicing():
     '''
-    Determine whether there are pending Component Based Servicing tasks that require a reboot.
-
-    :return: A boolean representing whether there are pending Component Based Servicing tasks.
-    :rtype: bool
+    Determine whether there are pending Component Based Servicing tasks that
+    require a reboot.
 
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1091,11 +1096,11 @@ def get_pending_domain_join():
     '''
     Determine whether there is a pending domain join action that requires a reboot.
 
-    :return: A boolean representing whether there is a pending domain join action.
-    :rtype: bool
-
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1130,13 +1135,14 @@ def get_pending_domain_join():
 
 def get_pending_file_rename():
     '''
-    Determine whether there are pending file rename operations that require a reboot.
-
-    :return: A boolean representing whether there are pending file rename operations.
-    :rtype: bool
+    Determine whether there are pending file rename operations that require a
+    reboot.
 
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1164,13 +1170,14 @@ def get_pending_file_rename():
 
 def get_pending_servermanager():
     '''
-    Determine whether there are pending Server Manager tasks that require a reboot.
-
-    :return: A boolean representing whether there are pending Server Manager tasks.
-    :rtype: bool
+    Determine whether there are pending Server Manager tasks that require a
+    reboot.
 
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1203,11 +1210,11 @@ def get_pending_update():
     '''
     Determine whether there are pending updates that require a reboot.
 
-    :return: A boolean representing whether there are pending updates.
-    :rtype: bool
-
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if a reboot is pending, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
@@ -1236,35 +1243,30 @@ REBOOT_REQUIRED_NAME = 'Reboot required'
 
 def set_reboot_required_witnessed():
     r'''
-    .. versionadded:: 2016.11.0
-
-    This function is used to remember that
-    an event indicating that a reboot is required was witnessed.
-    This function relies on the salt-minion's ability to create the following
-    volatile registry key in the *HKLM* hive:
+    This function is used to remember that an event indicating that a reboot is
+    required was witnessed. This function relies on the salt-minion's ability to
+    create the following volatile registry key in the *HKLM* hive:
 
        *SYSTEM\\CurrentControlSet\\Services\\salt-minion\\Volatile-Data*
 
-    Because this registry key is volatile, it will not persist
-    beyond the current boot session.
-    Also, in the scope of this key, the name *'Reboot required'* will be
-    assigned the value of *1*.
+    Because this registry key is volatile, it will not persist beyond the
+    current boot session. Also, in the scope of this key, the name *'Reboot
+    required'* will be assigned the value of *1*.
 
-    (For the time being, this this function is being used
-    whenever an install completes with exit code 3010 and
-    this usage can be extended where appropriate in the future.)
+    (For the time being, this this function is being used whenever an install
+    completes with exit code 3010 and this usage can be extended where
+    appropriate in the future.)
 
-    :return: A boolean indicating whether or not the salt minion was
-       able to perform the necessary registry operations.
+    .. versionadded:: 2016.11.0
 
-    :rtype: bool
+    Returns:
+        bool: True if registry entry set successfuly, otherwise False.
 
     CLI Example:
 
     .. code-block:: bash
 
         salt '*' system.set_reboot_required_witnessed
-
     '''
     return __salt__['reg.set_value'](hive='HKLM',
                                      key=MINION_VOLATILE_KEY,
@@ -1276,20 +1278,16 @@ def set_reboot_required_witnessed():
 
 def get_reboot_required_witnessed():
     '''
+    This tells us if, at any time during the current boot session the salt
+    minion witnessed an event indicating that a reboot is required. (For the
+    time being, this function will return True if an install completed with exit
+    code 3010 during the current boot session and this usage can be extended
+    where appropriate in the future)
+
     .. versionadded:: 2016.11.0
 
-    This tells us if, at any time during the current boot session
-    the salt minion witnessed an event indicating
-    that a reboot is required.
-    (For the time being, this function will return True
-    if an install completed with exit code 3010 during the current
-    boot session and this usage can be extended where appropriate
-    in the future)
-
-    :return: a boolean which will be True if the salt-minion reported
-       a required reboot during the current boot session, otherwise False.
-
-    :rtype: bool
+    Returns:
+        bool: True if reboot required, otherwise False.
 
     CLI Example:
 
@@ -1308,11 +1306,11 @@ def get_pending_reboot():
     '''
     Determine whether there is a reboot pending.
 
-    :return: A boolean representing whether reboots are pending.
-    :rtype: bool
-
     .. versionadded:: 2016.11.0
 
+    Returns:
+        bool: True if pending reboot, otherwise False.
+
     CLI Example:
 
     .. code-block:: bash
