@@ -19,7 +19,7 @@ import subprocess
 import jinja2
 import yaml
 import msgpack
-import salt.ext.six as six
+import salt.ext.six as _six
 import tornado
 
 # pylint: disable=import-error,no-name-in-module
@@ -54,11 +54,6 @@ except ImportError:
     # Older jinja does not need markupsafe
     HAS_MARKUPSAFE = False
 
-try:
-    import xml
-    HAS_XML = True
-except ImportError:
-    HAS_XML = False
 # pylint: enable=import-error,no-name-in-module
 
 try:
@@ -112,7 +107,7 @@ def get_tops(extra_mods='', so_mods=''):
             os.path.dirname(msgpack.__file__),
             ]
 
-    tops.append(six.__file__.replace('.pyc', '.py'))
+    tops.append(_six.__file__.replace('.pyc', '.py'))
 
     if HAS_CERTIFI:
         tops.append(os.path.dirname(certifi.__file__))
@@ -129,10 +124,6 @@ def get_tops(extra_mods='', so_mods=''):
     if HAS_SSL_MATCH_HOSTNAME:
         tops.append(os.path.dirname(os.path.dirname(ssl_match_hostname.__file__)))
 
-    if HAS_XML:
-        # For openSUSE, which apparently doesn't include the whole stdlib
-        tops.append(os.path.dirname(xml.__file__))
-
     for mod in [m for m in extra_mods.split(',') if m]:
         if mod not in locals() and mod not in globals():
             try:
@@ -163,7 +154,8 @@ def get_tops(extra_mods='', so_mods=''):
 
 
 def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
-             python2_bin='python2', python3_bin='python3'):
+             python2_bin='python2', python3_bin='python3', absonly=True,
+             compress='gzip'):
     '''
     Generate the salt-thin tarball and print the location of the tarball
     Optional additional mods to include (e.g. mako) can be supplied as a comma
@@ -181,7 +173,11 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
     thindir = os.path.join(cachedir, 'thin')
     if not os.path.isdir(thindir):
         os.makedirs(thindir)
-    thintar = os.path.join(thindir, 'thin.tgz')
+    if compress == 'gzip':
+        thin_ext = 'tgz'
+    elif compress == 'zip':
+        thin_ext = 'zip'
+    thintar = os.path.join(thindir, 'thin.' + thin_ext)
     thinver = os.path.join(thindir, 'version')
     pythinver = os.path.join(thindir, '.thin-gen-py-version')
     salt_call = os.path.join(thindir, 'salt-call')
@@ -205,7 +201,7 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
                 pass
         else:
             return thintar
-    if six.PY3:
+    if _six.PY3:
         # Let's check for the minimum python 2 version requirement, 2.6
         py_shell_cmd = (
             python2_bin + ' -c \'from __future__ import print_function; import sys; '
@@ -231,14 +227,14 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
 
     tops_py_version_mapping = {}
     tops = get_tops(extra_mods=extra_mods, so_mods=so_mods)
-    if six.PY2:
+    if _six.PY2:
         tops_py_version_mapping['2'] = tops
     else:
         tops_py_version_mapping['3'] = tops
 
     # TODO: Consider putting known py2 and py3 compatible libs in it's own sharable directory.
     #       This would reduce the thin size.
-    if six.PY2 and sys.version_info[0] == 2:
+    if _six.PY2 and sys.version_info[0] == 2:
         # Get python 3 tops
         py_shell_cmd = (
             python3_bin + ' -c \'import sys; import json; import salt.utils.thin; '
@@ -253,7 +249,7 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
                 tops_py_version_mapping['3'] = tops
             except ValueError:
                 pass
-    if six.PY3 and sys.version_info[0] == 3:
+    if _six.PY3 and sys.version_info[0] == 3:
         # Get python 2 tops
         py_shell_cmd = (
             python2_bin + ' -c \'from __future__ import print_function; '
@@ -270,14 +266,19 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
             except ValueError:
                 pass
 
-    tfp = tarfile.open(thintar, 'w:gz', dereference=True)
+    if compress == 'gzip':
+        tfp = tarfile.open(thintar, 'w:gz', dereference=True)
+    elif compress == 'zip':
+        tfp = zipfile.ZipFile(thintar, 'w')
     try:  # cwd may not exist if it was removed but salt was run from it
         start_dir = os.getcwd()
     except OSError:
         start_dir = None
     tempdir = None
-    for py_ver, tops in six.iteritems(tops_py_version_mapping):
+    for py_ver, tops in _six.iteritems(tops_py_version_mapping):
         for top in tops:
+            if absonly and not os.path.isabs(top):
+                continue
             base = os.path.basename(top)
             top_dirname = os.path.dirname(top)
             if os.path.isdir(top_dirname):
@@ -291,25 +292,43 @@ def gen_thin(cachedir, extra_mods='', overwrite=False, so_mods='',
                 os.chdir(tempdir)
             if not os.path.isdir(top):
                 # top is a single file module
-                tfp.add(base, arcname=os.path.join('py{0}'.format(py_ver), base))
+                if os.path.exists(os.path.join(top_dirname, base)):
+                    if compress == 'gzip':
+                        tfp.add(base, arcname=os.path.join('py{0}'.format(py_ver), base))
+                    elif compress == 'zip':
+                        tfp.write(base, arcname=os.path.join('py{0}'.format(py_ver), base))
                 continue
             for root, dirs, files in os.walk(base, followlinks=True):
                 for name in files:
                     if not name.endswith(('.pyc', '.pyo')):
-                        tfp.add(os.path.join(root, name),
-                                arcname=os.path.join('py{0}'.format(py_ver), root, name))
+                        if compress == 'gzip':
+                            tfp.add(os.path.join(root, name),
+                                    arcname=os.path.join('py{0}'.format(py_ver), root, name))
+                        elif compress == 'zip':
+                            try:
+                                # This is a little slow but there's no clear way to detect duplicates
+                                tfp.getinfo(os.path.join('py{0}'.format(py_ver), root, name))
+                            except KeyError:
+                                tfp.write(os.path.join(root, name), arcname=os.path.join('py{0}'.format(py_ver), root, name))
             if tempdir is not None:
                 shutil.rmtree(tempdir)
                 tempdir = None
     os.chdir(thindir)
-    tfp.add('salt-call')
+    if compress == 'gzip':
+        tfp.add('salt-call')
+    elif compress == 'zip':
+        tfp.write('salt-call')
     with salt.utils.fopen(thinver, 'w+') as fp_:
         fp_.write(salt.version.__version__)
     with salt.utils.fopen(pythinver, 'w+') as fp_:
         fp_.write(str(sys.version_info[0]))
     os.chdir(os.path.dirname(thinver))
-    tfp.add('version')
-    tfp.add('.thin-gen-py-version')
+    if compress == 'gzip':
+        tfp.add('version')
+        tfp.add('.thin-gen-py-version')
+    elif compress == 'zip':
+        tfp.write('version')
+        tfp.write('.thin-gen-py-version')
     if start_dir:
         os.chdir(start_dir)
     tfp.close()
@@ -367,7 +386,7 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
                 pass
         else:
             return mintar
-    if six.PY3:
+    if _six.PY3:
         # Let's check for the minimum python 2 version requirement, 2.6
         py_shell_cmd = (
             python2_bin + ' -c \'from __future__ import print_function; import sys; '
@@ -393,14 +412,14 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
 
     tops_py_version_mapping = {}
     tops = get_tops(extra_mods=extra_mods, so_mods=so_mods)
-    if six.PY2:
+    if _six.PY2:
         tops_py_version_mapping['2'] = tops
     else:
         tops_py_version_mapping['3'] = tops
 
     # TODO: Consider putting known py2 and py3 compatible libs in it's own sharable directory.
     #       This would reduce the min size.
-    if six.PY2 and sys.version_info[0] == 2:
+    if _six.PY2 and sys.version_info[0] == 2:
         # Get python 3 tops
         py_shell_cmd = (
             python3_bin + ' -c \'import sys; import json; import salt.utils.thin; '
@@ -415,7 +434,7 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
                 tops_py_version_mapping['3'] = tops
             except ValueError:
                 pass
-    if six.PY3 and sys.version_info[0] == 3:
+    if _six.PY3 and sys.version_info[0] == 3:
         # Get python 2 tops
         py_shell_cmd = (
             python2_bin + ' -c \'from __future__ import print_function; '
@@ -556,7 +575,7 @@ def gen_min(cachedir, extra_mods='', overwrite=False, so_mods='',
         'salt/output/nested.py',
     )
 
-    for py_ver, tops in six.iteritems(tops_py_version_mapping):
+    for py_ver, tops in _six.iteritems(tops_py_version_mapping):
         for top in tops:
             base = os.path.basename(top)
             top_dirname = os.path.dirname(top)
