@@ -2,6 +2,7 @@
 '''
 A few checks to make sure the environment is sane
 '''
+from __future__ import absolute_import
 
 # Original Author: Jeff Schroeder <jeffschroeder@computer.org>
 
@@ -10,19 +11,22 @@ import os
 import re
 import sys
 import stat
+import errno
 import socket
 import logging
 
 # Import third party libs
-if sys.platform.startswith('win'):
+try:
     import win32file
-else:
+except ImportError:
     import resource
 
 # Import salt libs
 from salt.log import is_console_configured
-from salt.exceptions import SaltClientError
-import salt.exitcodes
+from salt.log.setup import LOG_LEVELS
+from salt.exceptions import SaltClientError, SaltSystemExit, \
+    CommandExecutionError
+import salt.defaults.exitcodes
 import salt.utils
 
 log = logging.getLogger(__name__)
@@ -46,7 +50,7 @@ def zmq_version():
     if not match:
         msg = "Using untested zmq python bindings version: '{0}'".format(ver)
         if is_console_configured():
-            log.warn(msg)
+            log.warning(msg)
         else:
             sys.stderr.write("WARNING {0}\n".format(msg))
         return True
@@ -67,7 +71,7 @@ def zmq_version():
         if "dev" in ver and not point:
             msg = 'Using dev zmq module, please report unexpected results'
             if is_console_configured():
-                log.warn(msg)
+                log.warning(msg)
             else:
                 sys.stderr.write("WARNING: {0}\n".format(msg))
             return True
@@ -114,32 +118,27 @@ def verify_socket(interface, pub_port, ret_port):
     '''
 
     addr_family = lookup_family(interface)
-    pubsock = socket.socket(addr_family, socket.SOCK_STREAM)
-    retsock = socket.socket(addr_family, socket.SOCK_STREAM)
-    try:
-        pubsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        pubsock.bind((interface, int(pub_port)))
-        pubsock.close()
-        retsock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
-        retsock.bind((interface, int(ret_port)))
-        retsock.close()
-        result = True
-    except Exception as exc:
-        if exc.args:
-            msg = ('Unable to bind socket, error: {0}'.format(str(exc)))
-        else:
-            msg = ('Unable to bind socket, this might not be a problem.'
-                   ' Is there another salt-master running?')
-        if is_console_configured():
-            log.warn(msg)
-        else:
-            sys.stderr.write('WARNING: {0}\n'.format(msg))
-        result = False
-    finally:
-        pubsock.close()
-        retsock.close()
+    for port in pub_port, ret_port:
+        sock = socket.socket(addr_family, socket.SOCK_STREAM)
+        try:
+            sock.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
+            sock.bind((interface, int(port)))
+        except Exception as exc:
+            msg = 'Unable to bind socket {0}:{1}'.format(interface, port)
+            if exc.args:
+                msg = '{0}, error: {1}'.format(msg, str(exc))
+            else:
+                msg = '{0}, this might not be a problem.'.format(msg)
+            msg += '; Is there another salt-master running?'
+            if is_console_configured():
+                log.warning(msg)
+            else:
+                sys.stderr.write('WARNING: {0}\n'.format(msg))
+            return False
+        finally:
+            sock.close()
 
-    return result
+    return True
 
 
 def verify_files(files, user):
@@ -152,24 +151,37 @@ def verify_files(files, user):
     try:
         pwnam = pwd.getpwnam(user)
         uid = pwnam[2]
-
     except KeyError:
         err = ('Failed to prepare the Salt environment for user '
                '{0}. The user is not available.\n').format(user)
         sys.stderr.write(err)
-        sys.exit(salt.exitcodes.EX_NOUSER)
+        sys.exit(salt.defaults.exitcodes.EX_NOUSER)
+
     for fn_ in files:
         dirname = os.path.dirname(fn_)
         try:
-            if not os.path.isdir(dirname):
-                os.makedirs(dirname)
+            if dirname:
+                try:
+                    os.makedirs(dirname)
+                except OSError as err:
+                    if err.errno != errno.EEXIST:
+                        raise
             if not os.path.isfile(fn_):
                 with salt.utils.fopen(fn_, 'w+') as fp_:
                     fp_.write('')
+
+        except IOError as err:
+            if os.path.isfile(dirname):
+                msg = 'Failed to create path {0}, is {1} a file?'.format(fn_, dirname)
+                raise SaltSystemExit(msg=msg)
+            if err.errno != errno.EACCES:
+                raise
+            msg = 'No permissions to access "{0}", are you running as the correct user?'.format(fn_)
+            raise SaltSystemExit(msg=msg)
+
         except OSError as err:
-            msg = 'Failed to create path "{0}" - {1}\n'
-            sys.stderr.write(msg.format(fn_, err))
-            sys.exit(err.errno)
+            msg = 'Failed to create path "{0}" - {1}'.format(fn_, err)
+            raise SaltSystemExit(msg=msg)
 
         stats = os.stat(fn_)
         if uid != stats.st_uid:
@@ -180,13 +192,13 @@ def verify_files(files, user):
     return True
 
 
-def verify_env(dirs, user, permissive=False, pki_dir=''):
+def verify_env(dirs, user, permissive=False, pki_dir='', skip_extra=False):
     '''
     Verify that the named directories are in place and that the environment
     can shake the salt
     '''
     if salt.utils.is_windows():
-        return True
+        return win_verify_env(dirs, permissive, pki_dir, skip_extra)
     import pwd  # after confirming not running Windows
     try:
         pwnam = pwd.getpwnam(user)
@@ -198,7 +210,7 @@ def verify_env(dirs, user, permissive=False, pki_dir=''):
         err = ('Failed to prepare the Salt environment for user '
                '{0}. The user is not available.\n').format(user)
         sys.stderr.write(err)
-        sys.exit(salt.exitcodes.EX_NOUSER)
+        sys.exit(salt.defaults.exitcodes.EX_NOUSER)
     for dir_ in dirs:
         if not dir_:
             continue
@@ -262,7 +274,7 @@ def verify_env(dirs, user, permissive=False, pki_dir=''):
         #
         # If the permissions aren't correct, default to the more secure 700.
         # If acls are enabled, the pki_dir needs to remain readable, this
-        # is still secure because the private keys are still only readbale
+        # is still secure because the private keys are still only readable
         # by the user running the master
         if dir_ == pki_dir:
             smode = stat.S_IMODE(mode.st_mode)
@@ -276,8 +288,10 @@ def verify_env(dirs, user, permissive=False, pki_dir=''):
                         log.critical(msg)
                     else:
                         sys.stderr.write("CRITICAL: {0}\n".format(msg))
-    # Run the extra verification checks
-    zmq_version()
+
+    if skip_extra is False:
+        # Run the extra verification checks
+        zmq_version()
 
 
 def check_user(user):
@@ -293,12 +307,24 @@ def check_user(user):
         pwuser = pwd.getpwnam(user)
         try:
             if hasattr(os, 'initgroups'):
-                os.initgroups(user, pwuser.pw_gid)
+                os.initgroups(user, pwuser.pw_gid)  # pylint: disable=minimum-python-version
             else:
                 os.setgroups(salt.utils.get_gid_list(user, include_default=False))
             os.setgid(pwuser.pw_gid)
             os.setuid(pwuser.pw_uid)
 
+            # We could just reset the whole environment but let's just override
+            # the variables we can get from pwuser
+            if 'HOME' in os.environ:
+                os.environ['HOME'] = pwuser.pw_dir
+
+            if 'SHELL' in os.environ:
+                os.environ['SHELL'] = pwuser.pw_shell
+
+            for envvar in ('USER', 'LOGNAME'):
+                if envvar in os.environ:
+                    os.environ[envvar] = pwuser.pw_name
+
         except OSError:
             msg = 'Salt configured to run as user "{0}" but unable to switch.'
             msg = msg.format(user)
@@ -387,7 +413,7 @@ def check_max_open_files(opts):
         mof_s, mof_h = resource.getrlimit(resource.RLIMIT_NOFILE)
 
     accepted_keys_dir = os.path.join(opts.get('pki_dir'), 'minions')
-    accepted_count = sum(1 for _ in os.listdir(accepted_keys_dir))
+    accepted_count = len(os.listdir(accepted_keys_dir))
 
     log.debug(
         'This salt-master instance has accepted {0} minion keys.'.format(
@@ -454,13 +480,22 @@ def clean_path(root, path, subdir=False):
     return ''
 
 
+def clean_id(id_):
+    '''
+    Returns if the passed id is clean.
+    '''
+    if re.search(r'\.\.\{sep}'.format(sep=os.sep), id_):
+        return False
+    return True
+
+
 def valid_id(opts, id_):
     '''
     Returns if the passed id is valid
     '''
     try:
-        return bool(clean_path(opts['pki_dir'], id_))
-    except (AttributeError, KeyError) as e:
+        return bool(clean_path(opts['pki_dir'], id_)) and clean_id(id_)
+    except (AttributeError, KeyError, TypeError) as e:
         return False
 
 
@@ -483,3 +518,121 @@ def safe_py_code(code):
         if code.count(bad):
             return False
     return True
+
+
+def verify_log(opts):
+    '''
+    If an insecre logging configuration is found, show a warning
+    '''
+    level = LOG_LEVELS.get(str(opts.get('log_level')).lower(), logging.NOTSET)
+
+    if level < logging.INFO:
+        log.warning('Insecure logging configuration detected! Sensitive data may be logged.')
+
+
+def win_verify_env(dirs, permissive=False, pki_dir='', skip_extra=False):
+    '''
+    Verify that the named directories are in place and that the environment
+    can shake the salt
+    '''
+    import salt.utils.win_functions
+    import salt.utils.win_dacl
+
+    # Get the root path directory where salt is installed
+    path = dirs[0]
+    while os.path.basename(path) not in ['salt', 'salt-tests-tmpdir']:
+        path, base = os.path.split(path)
+
+    # Create the root path directory if missing
+    if not os.path.isdir(path):
+        os.makedirs(path)
+
+    # Set permissions to the root path directory
+    current_user = salt.utils.win_functions.get_current_user()
+    if salt.utils.win_functions.is_admin(current_user):
+        try:
+            # Make the Administrators group owner
+            # Use the SID to be locale agnostic
+            salt.utils.win_dacl.set_owner(path, 'S-1-5-32-544')
+
+        except CommandExecutionError:
+            msg = 'Unable to securely set the owner of "{0}".'.format(path)
+            if is_console_configured():
+                log.critical(msg)
+            else:
+                sys.stderr.write("CRITICAL: {0}\n".format(msg))
+
+        if not permissive:
+            try:
+                # Get a clean dacl by not passing an obj_name
+                dacl = salt.utils.win_dacl.dacl()
+
+                # Add aces to the dacl, use the GUID (locale non-specific)
+                # Administrators Group
+                dacl.add_ace('S-1-5-32-544', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # System
+                dacl.add_ace('S-1-5-18', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # Owner
+                dacl.add_ace('S-1-3-4', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+
+                # Save the dacl to the object
+                dacl.save(path, True)
+
+            except CommandExecutionError:
+                msg = 'Unable to securely set the permissions of ' \
+                      '"{0}".'.format(path)
+                if is_console_configured():
+                    log.critical(msg)
+                else:
+                    sys.stderr.write("CRITICAL: {0}\n".format(msg))
+
+    # Create the directories
+    for dir_ in dirs:
+        if not dir_:
+            continue
+        if not os.path.isdir(dir_):
+            try:
+                os.makedirs(dir_)
+            except OSError as err:
+                msg = 'Failed to create directory path "{0}" - {1}\n'
+                sys.stderr.write(msg.format(dir_, err))
+                sys.exit(err.errno)
+
+        # The PKI dir gets its own permissions
+        if dir_ == pki_dir:
+            try:
+                # Make Administrators group the owner
+                salt.utils.win_dacl.set_owner(path, 'S-1-5-32-544')
+
+                # Give Admins, System and Owner permissions
+                # Get a clean dacl by not passing an obj_name
+                dacl = salt.utils.win_dacl.dacl()
+
+                # Add aces to the dacl, use the GUID (locale non-specific)
+                # Administrators Group
+                dacl.add_ace('S-1-5-32-544', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # System
+                dacl.add_ace('S-1-5-18', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+                # Owner
+                dacl.add_ace('S-1-3-4', 'grant', 'full_control',
+                             'this_folder_subfolders_files')
+
+                # Save the dacl to the object
+                dacl.save(dir_, True)
+
+            except CommandExecutionError:
+                msg = 'Unable to securely set the permissions of "{0}".'
+                msg = msg.format(dir_)
+                if is_console_configured():
+                    log.critical(msg)
+                else:
+                    sys.stderr.write("CRITICAL: {0}\n".format(msg))
+
+    if skip_extra is False:
+        # Run the extra verification checks
+        zmq_version()
